#![no_std]
#![no_main]

use core::cell::Cell;

use ble::gatt::Server;
use cortex_m::interrupt::Mutex;
use defmt_rtt as _;
use embassy_nrf as _;
use embassy_sync::blocking_mutex::raw::CriticalSectionRawMutex;
use embassy_time::Delay;
use embedded_hal_async::delay::DelayNs;
use flash::Config;
use futures::pin_mut;
use nrf_softdevice::{Flash, Softdevice};
use panic_probe as _;

use crate::lcd::Lcd;
use defmt::{error, info, unwrap};
use embassy_executor::Spawner;

pub mod ble {
    pub mod advertisment;
    pub mod battery_service;
    pub mod config_service;
    pub mod gatt;
}
pub mod flash;
pub mod lcd;
pub mod peripherals;
pub mod softdevice;

const DEVICE_NAME: &str = "alert-me display";

#[embassy_executor::task]
async fn ble(sd: &'static Softdevice, server: Server) {
    loop {
        let conn = ble::advertisment::run(sd, DEVICE_NAME).await;
        let error = ble::gatt::run(&conn, &server).await;
        info!("gatt_server run exited with error: {:?}", error);
    }
}

#[embassy_executor::task]
async fn tick(sd: &'static Softdevice, mut lcd: lcd::Lcd<'static>, delay: u32) {
    let mut d = Delay {};
    loop {
        lcd.scroll_buffer()
            .await
            .unwrap_or_else(|e| error!("Failed to scroll lcd: {}", e));

        d.delay_ms(delay).await;
    }
}

#[embassy_executor::main]
async fn main(spawner: Spawner) {
    let peripherals = peripherals::init();
    let mut lcd = Lcd::new(
        0x27,
        peripherals.TWISPI0,
        peripherals.P0_03,
        peripherals.P0_04,
    );
    lcd.init().await.expect("Failed to initialize lcd");
    let (sd, server) = softdevice::run(spawner, DEVICE_NAME).expect("Failed to run softdevice");

    let flash = Flash::take(sd);
    pin_mut!(flash);
    let config = flash::Config::new(flash);

    unwrap!(spawner.spawn(ble(sd, server)));
    unwrap!(spawner.spawn(tick(sd, lcd, 500)));
}
